@@ -70,6 +70,7 @@ let dump (type a) : a t -> json =
         ("position", mk_position pos)
       ]
   | Locate_type pos -> mk "locate-type" [ ("position", mk_position pos) ]
+  | Locate_types pos -> mk "locate-types" [ ("position", mk_position pos) ]
   | Enclosing pos -> mk "enclosing" [ ("position", mk_position pos) ]
   | Complete_prefix (prefix, pos, kind, doc, typ) ->
     mk "complete-prefix"
@@ -134,12 +135,19 @@ let dump (type a) : a t -> json =
           | Some `Local -> `String "local" );
         ("depth", `Int depth)
       ]
-  | Inlay_hints (start, stop, hint_let_binding, hint_pattern_var, ghost) ->
+  | Inlay_hints
+      ( start,
+        stop,
+        hint_let_binding,
+        hint_pattern_var,
+        hint_function_params,
+        ghost ) ->
     mk "inlay-hints"
       [ ("start", mk_position start);
         ("stop", mk_position stop);
         ("hint-let-binding", `Bool hint_let_binding);
         ("hint-pattern-variable", `Bool hint_pattern_var);
+        ("hint-function-params", `Bool hint_function_params);
         ("avoid-ghost-location", `Bool ghost)
       ]
   | Outline -> mk "outline" []
@@ -220,6 +228,7 @@ let string_of_completion_kind = function
   | `MethodCall -> "#"
   | `Exn -> "Exn"
   | `Class -> "Class"
+  | `ClassType -> "ClassType"
   | `Keyword -> "Keyword"
 
 let with_location ?(with_file = false) ?(skip_none = false) loc assoc =
@@ -315,6 +324,7 @@ let rec json_of_outline outline =
         outline_kind;
         outline_type;
         location;
+        selection;
         children;
         deprecated
       } =
@@ -326,7 +336,8 @@ let rec json_of_outline outline =
           | None -> `Null
           | Some typ -> `String typ );
         ("children", `List (json_of_outline children));
-        ("deprecated", `Bool deprecated)
+        ("deprecated", `Bool deprecated);
+        ("selection", with_location selection [])
       ]
   in
   List.map ~f:json_of_item outline
@@ -354,6 +365,64 @@ let json_of_locate resp =
   | `Found (Some file, pos) ->
     `Assoc [ ("file", `String file); ("pos", Lexing.json_of_position pos) ]
 
+let json_of_locate_types (resp : Locate_types_result.t) =
+  (* This function serializes to json differently than json_of_locate to make it easier
+     to deserialize. *)
+  let json_of_position
+      ({ pos_fname; pos_lnum; pos_bol; pos_cnum } : Lexing.position) =
+    `Assoc
+      [ ("pos_fname", `String pos_fname);
+        ("pos_lnum", `Int pos_lnum);
+        ("pos_bol", `Int pos_bol);
+        ("pos_cnum", `Int pos_cnum)
+      ]
+  in
+  let json_of_option opt ~f =
+    match opt with
+    | None -> `Null
+    | Some x -> f x
+  in
+  let json_of_locate_result result =
+    let variant_name, payload =
+      match result with
+      | `Found (file, pos) ->
+        let file = json_of_option file ~f:(fun file -> `String file) in
+        ("Found", [ file; json_of_position pos ])
+      | `File_not_found msg -> ("File_not_found", [ `String msg ])
+      | `Builtin id -> ("Builtin", [ `String id ])
+      | `Not_found (id, file) ->
+        let file = json_of_option file ~f:(fun file -> `String file) in
+        ("Not_found", [ `String id; file ])
+      | `Not_in_env id -> ("Not_in_env", [ `String id ])
+    in
+    `List (`String variant_name :: payload)
+  in
+  let json_of_node_data :
+      Locate_types_result.(type_ref_payload Tree.node_data) ->
+      _ = function
+    | Arrow -> `List [ `String "Arrow" ]
+    | Tuple -> `List [ `String "Tuple" ]
+    | Object -> `List [ `String "Object" ]
+    | Poly_variant -> `List [ `String "Poly_variant" ]
+    | Type_ref { type_; result } ->
+      `List
+        [ `String "Type_ref";
+          `Assoc
+            [ ("type", `String type_);
+              ("result", json_of_locate_result result)
+            ]
+        ]
+  in
+  let rec json_of_type_tree { Locate_types_result.Tree.data; children } =
+    `Assoc
+      [ ("data", json_of_node_data data);
+        ("children", `List (List.map ~f:json_of_type_tree children))
+      ]
+  in
+  match resp with
+  | Success tree -> json_of_type_tree tree
+  | Invalid_context -> `String "Invalid context"
+
 let json_of_inlay_hints hints =
   let json_of_hint (position, label) =
     `Assoc
@@ -451,6 +520,7 @@ let json_of_response (type a) (query : a t) (response : a) : json =
     in
     str
   | Locate_type _, resp -> json_of_locate resp
+  | Locate_types _, resp -> json_of_locate_types resp
   | Locate _, resp -> json_of_locate resp
   | Jump _, resp -> begin
     match resp with
@@ -488,8 +558,11 @@ let json_of_response (type a) (query : a t) (response : a) : json =
   | Findlib_list, strs -> `List (List.map ~f:Json.string strs)
   | Extension_list _, strs -> `List (List.map ~f:Json.string strs)
   | Path_list _, strs -> `List (List.map ~f:Json.string strs)
-  | Occurrences (_, scope), (locations, _project) ->
+  | Occurrences (_, scope), (occurrences, _project) ->
     let with_file = scope = `Project || scope = `Renaming in
-    `List (List.map locations ~f:(fun loc -> with_location ~with_file loc []))
+    `List
+      (List.map occurrences ~f:(fun occurrence ->
+           with_location ~with_file occurrence.loc
+             [ ("stale", Json.bool occurrence.is_stale) ]))
   | Signature_help _, s -> json_of_signature_help s
   | Version, version -> `String version
