@@ -349,7 +349,7 @@ let of_method_call obj meth loc env (f : _ f0) acc =
 let rec of_expression_desc loc = function
   | Texp_ident _ | Texp_constant _ | Texp_instvar _
   | Texp_variant (_, None)
-  | Texp_new _ | Texp_hole -> id_fold
+  | Texp_new _ | Texp_typed_hole -> id_fold
   | Texp_let (_, vbs, e) -> of_expression e ** list_fold of_value_binding vbs
   | Texp_function (params, body) ->
     list_fold of_function_param params ** of_function_body body
@@ -474,7 +474,7 @@ and of_module_expr_desc = function
   | Tmod_constraint (me, _, mtc, _) ->
     of_module_expr me ** app (Module_type_constraint mtc)
   | Tmod_unpack (e, _) -> of_expression e
-  | Tmod_hole -> id_fold
+  | Tmod_typed_hole -> id_fold
 
 and of_structure_item_desc = function
   | Tstr_eval (e, _) -> of_expression e
@@ -933,9 +933,10 @@ let all_holes (env, node) =
   let rec aux acc (env, node) =
     let f env node acc =
       match node with
-      | Expression { exp_desc = Texp_hole; exp_loc; exp_type; exp_env; _ } ->
-        (exp_loc, exp_env, `Exp exp_type) :: acc
-      | Module_expr { mod_desc = Tmod_hole; mod_loc; mod_type; mod_env; _ } ->
+      | Expression { exp_desc = Texp_typed_hole; exp_loc; exp_type; exp_env; _ }
+        -> (exp_loc, exp_env, `Exp exp_type) :: acc
+      | Module_expr
+          { mod_desc = Tmod_typed_hole; mod_loc; mod_type; mod_env; _ } ->
         (mod_loc, mod_env, `Mod mod_type) :: acc
       | _ -> aux acc (env, node)
     in
