@@ -84,10 +84,10 @@ val input_lexbuf: Lexing.lexbuf option ref
    toplevel phrase. *)
 val input_phrase_buffer: Buffer.t option ref
 
+
 (** {1 Toplevel-specific functions} *)
 
 val echo_eof: unit -> unit
-val separate_new_message: formatter -> unit
 val reset: unit -> unit
 
 
@@ -173,11 +173,20 @@ val show_filename: string -> string
         Otherwise, returns the filename unchanged. *)
 
 val print_filename: formatter -> string -> unit
-
 val print_loc: formatter -> t -> unit
 val print_locs: formatter -> t list -> unit
+val separate_new_message: formatter -> unit
+
+module Doc: sig
+  val separate_new_message: unit Format_doc.printer
+  val filename: string Format_doc.printer
+  val quoted_filename: string Format_doc.printer
+  val loc: t Format_doc.printer
+  val locs: t list Format_doc.printer
+end
 
 (** {1 Toplevel-specific location highlighting} *)
+
 (*
 val highlight_terminfo:
   Lexing.lexbuf -> formatter -> t list -> unit
@@ -187,9 +196,9 @@ val highlight_terminfo:
 
 (** {2 The type of reports and report printers} *)
 
-type msg = (Format.formatter -> unit) loc
+type msg = Format_doc.t loc
 
-val msg: ?loc:t -> ('a, Format.formatter, unit, msg) format4 -> 'a
+val msg: ?loc:t -> ('a, Format_doc.formatter, unit, msg) format4 -> 'a
 
 type report_kind =
   | Report_error
@@ -204,9 +213,11 @@ type report = {
   kind : report_kind;
   main : msg;
   sub : msg list;
+  footnote: Format_doc.t option;
   source : error_source;
 }
 
+
 (* Exposed for Merlin *)
 val loc_of_report: report -> t
 val print_main : formatter -> report -> unit
@@ -222,7 +233,7 @@ type report_printer = {
   pp_main_loc : report_printer -> report ->
     Format.formatter -> t -> unit;
   pp_main_txt : report_printer -> report ->
-    Format.formatter -> (Format.formatter -> unit) -> unit;
+    Format.formatter -> Format_doc.t -> unit;
   pp_submsgs : report_printer -> report ->
     Format.formatter -> msg list -> unit;
   pp_submsg : report_printer -> report ->
@@ -230,7 +241,7 @@ type report_printer = {
   pp_submsg_loc : report_printer -> report ->
     Format.formatter -> t -> unit;
   pp_submsg_txt : report_printer -> report ->
-    Format.formatter -> (Format.formatter -> unit) -> unit;
+    Format.formatter -> Format_doc.t -> unit;
 }
 (** A printer for [report]s, defined using open-recursion.
     The goal is to make it easy to define new printers by re-using code from
@@ -240,6 +251,7 @@ type report_printer = {
 (** {2 Report printers used in the compiler} *)
 
 val batch_mode_printer: report_printer
+
 (*
 val terminfo_toplevel_printer: Lexing.lexbuf -> report_printer
 
@@ -309,7 +321,7 @@ val default_alert_reporter: t -> Warnings.alert -> report option
 
 val print_alert: t -> formatter -> Warnings.alert -> unit
 (** Prints an alert. This is simply the composition of [report_alert] and
-   [print_report]. *)
+    [print_report]. *)
 
 val prerr_alert_ref: (t -> Warnings.alert -> unit) ref
 
@@ -336,15 +348,19 @@ val deprecated_script_alert: string -> unit
 type error = report
 (** An [error] is a [report] which [report_kind] must be [Report_error]. *)
 
-val error: ?loc:t -> ?sub:msg list -> ?source:error_source -> string -> error
+type delayed_msg = unit -> Format_doc.t option
 
-val errorf: ?loc:t -> ?sub:msg list -> ?source:error_source ->
-  ('a, Format.formatter, unit, error) format4 -> 'a
+val error: ?loc:t -> ?sub:msg list -> ?footnote:delayed_msg ->
+  ?source:error_source -> string -> error
 
-val error_of_printer: ?loc:t -> ?sub:msg list -> ?source:error_source ->
-  (formatter -> 'a -> unit) -> 'a -> error
+val errorf: ?loc:t -> ?sub:msg list -> ?footnote:delayed_msg ->
+  ?source:error_source -> ('a, Format_doc.formatter, unit, error) format4 -> 'a
 
-val error_of_printer_file: ?source:error_source -> (formatter -> 'a -> unit) -> 'a -> error
+val error_of_printer: ?loc:t -> ?sub:msg list -> ?footnote:delayed_msg ->
+  ?source:error_source -> (Format_doc.formatter -> 'a -> unit) -> 'a -> error
+
+val error_of_printer_file: ?source:error_source ->
+  (Format_doc.formatter -> 'a -> unit) -> 'a -> error
 
 
 (** {1 Automatically reporting errors for raised exceptions} *)
@@ -367,8 +383,8 @@ exception Already_displayed_error
 (** Raising [Already_displayed_error] signals an error which has already been
    printed. The exception will be caught, but nothing will be printed *)
 
-val raise_errorf: ?loc:t -> ?sub:msg list -> ?source:error_source ->
-  ('a, Format.formatter, unit, 'b) format4 -> 'a
+val raise_errorf: ?loc:t -> ?sub:msg list -> ?footnote:delayed_msg ->
+  ?source:error_source -> ('a, Format_doc.formatter, unit, 'b) format4 -> 'a
 
 val report_exception: formatter -> exn -> unit
 (** Reraise the exception if it is unknown. *)
