@@ -149,7 +149,7 @@ and expression_desc =
   | Texp_unreachable
   | Texp_extension_constructor of Longident.t loc * Path.t
   | Texp_open of open_declaration * expression
-  | Texp_hole
+  | Texp_typed_hole
 
 and meth =
   | Tmeth_name of string
@@ -285,7 +285,7 @@ and module_expr_desc =
   | Tmod_constraint of
       module_expr * Types.module_type * module_type_constraint * module_coercion
   | Tmod_unpack of expression * Types.module_type
-  | Tmod_hole
+  | Tmod_typed_hole
 
 and structure = {
   str_items : structure_item list;
